﻿using System;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;

using Windows.ApplicationModel.Background;
using Windows.UI.Notifications;
using Windows.Data.Xml.Dom;
using Windows.Foundation;
using Windows.Storage;
using Windows.System.Threading;
using Windows.Networking;
using Windows.Networking.Connectivity;

namespace BackgroundWifiCheck
{
    public sealed class TheTask : IBackgroundTask
    {
        class Away
        {
            public string away { get; set; }
        }

        public async void Run(IBackgroundTaskInstance taskInstance)
        {
            var deferral = taskInstance.GetDeferral();

            var settings = ApplicationData.Current.LocalSettings;
            var key = "theCurrentWifi";

            var ssid = "";
            var checkHomeWifi = "";
            foreach (var connectionProfile in NetworkInformation.GetConnectionProfiles())
            {
                if (connectionProfile.IsWlanConnectionProfile && connectionProfile.GetNetworkConnectivityLevel() != NetworkConnectivityLevel.None)
                {
                    ssid = connectionProfile.ProfileName;
                    break;
                }
            }
            settings.Values[key] = ssid;

            if (settings.Values.ContainsKey("nameHomeWifi"))
            {
                checkHomeWifi = settings.Values["nameHomeWifi"].ToString();
            }
            else 
            { 
                checkHomeWifi = ""; 
            }

            var t = Task.Run(async delegate
            {
                await Task.Delay(TimeSpan.FromSeconds(5));
                return 0;
            });
            t.Wait();

            string homeStatus = " --- ";
            string authToken = "-- insert your Nest token Here ---";
            string uriNest = "https://developer-api.nest.com/structures/?auth=" + authToken;
            try
            {
                var client = new HttpClient();
                var uri = new Uri(uriNest);
                var Response = await client.GetAsync(uri);
                var statusCode = Response.StatusCode;

                Response.EnsureSuccessStatusCode();
                var ResponseText = await Response.Content.ReadAsStringAsync();
                string json = ResponseText;
                int structureFirst = json.IndexOf("{\"") + 2;
                int structureNext = json.IndexOf("\":{\"name\"");
                string structure = json.Substring(structureFirst, structureNext - structureFirst);

                int awayFirst = json.IndexOf("\"away\":") + 8;
                int awayNext = json.IndexOf("\",\"", awayFirst);
                string away = json.Substring(awayFirst, awayNext - awayFirst);
                homeStatus = "Already: " + away;

                Away setAway;
                uriNest = "https://developer-api.nest.com/structures/" + structure + "/?auth=" + authToken;

                if (checkHomeWifi == ssid)
                {
                    if (away == "away")
                    {
                        setAway = new Away() { away = "home" };
                        HttpResponseMessage response = await client.PutAsJsonAsync(uriNest, setAway);
                        if (response.IsSuccessStatusCode)
                        {
                            homeStatus = "Home - Success!";
                        }
                        else
                        {
                            homeStatus = "Home - Fail!";
                        }
                    }
                }
                else
                {
                    if (away == "home")
                    {
                        setAway = new Away() { away = "away" };
                        HttpResponseMessage response = await client.PutAsJsonAsync(uriNest, setAway);
                        if (response.IsSuccessStatusCode)
                        {
                            homeStatus = "Away - Success!";
                        }
                        else 
                        {
                            homeStatus = "Away - Fail!";
                        }
                    }
                 }

            }
            catch (Exception ex)
            {
                homeStatus = ex.ToString();
            }

            ToastTemplateType toastTemplate = ToastTemplateType.ToastText02;
            XmlDocument toastXml = ToastNotificationManager.GetTemplateContent(toastTemplate);
            XmlNodeList textElements = toastXml.GetElementsByTagName("text");
            textElements[0].AppendChild(toastXml.CreateTextNode("Nest Wifi change alert!"));
            textElements[1].AppendChild(toastXml.CreateTextNode(homeStatus));

            ToastNotificationManager.CreateToastNotifier().Show(new ToastNotification(toastXml));

            taskInstance.Canceled += (s, e) => { };
            taskInstance.Progress = 0;
            deferral.Complete();
        }
    }
}
